<?php
/**
 * Plugin Name: Blog Card Block
 * Plugin URI: https://dodon.me/plugin/blog-card-block/
 * Description: 投稿にリンクを張り付けるだけでブログカードを作成できるブロックを追加します。
 * Version: 1.0.0
 * Author: dodon
 * Author URI: https://dodon.me/
 * License: GPL-2.0-or-later
 * Text Domain: blog-card-block
 */

if (!defined('ABSPATH')) {
    exit;
}

define('BCB_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('BCB_PLUGIN_URL', plugin_dir_url(__FILE__));
define('BCB_VERSION', '1.0.1');

function bcb_block_init() {

    $settings = get_option('bcb_settings', array());


    wp_register_script(
        'blog-card-block-editor',
        BCB_PLUGIN_URL . 'build/index.js',
        array('wp-blocks', 'wp-element', 'wp-block-editor', 'wp-components', 'wp-i18n', 'wp-api-fetch'),
        BCB_VERSION
    );


    wp_register_style(
        'blog-card-block-style',
        BCB_PLUGIN_URL . 'build/style-index.css',
        array(),
        BCB_VERSION
    );


    wp_register_style(
        'blog-card-block-editor-style',
        BCB_PLUGIN_URL . 'editor-style.css', 
        array('wp-edit-blocks'),
        BCB_VERSION
    );


    if (function_exists('wp_set_script_translations')) {
        wp_set_script_translations('blog-card-block-editor', 'blog-card-block');
    }


    wp_localize_script('blog-card-block-editor', 'bcbSettings', $settings);


    register_block_type('blog-card-block/blog-card', array(
        'api_version' => 2,
        'editor_script' => 'blog-card-block-editor',
        'editor_style' => 'blog-card-block-editor-style',
        'style' => 'blog-card-block-style',
        'title' => 'ブログカード',
        'category' => 'widgets',
        'icon' => 'admin-links',
        'description' => 'リンク先の情報を取得し、ブログカードとして表示します。',
        'keywords' => array('ブログカード', 'リンクカード', 'リンク', 'カード', 'blog card', 'link card'),
        'supports' => array(
            'html' => false,
        ),
        'render_callback' => 'bcb_render_callback',
        'attributes' => array(
            'url' => array(
                'type' => 'string',
                'default' => '',
            ),
            'cardData' => array(
                'type' => 'object',
                'default' => null,
            ),
        ),
    ));
}
add_action('init', 'bcb_block_init');


function bcb_add_block_categories($categories) {
    return array_merge(
        $categories,
        array(
            array(
                'slug' => 'blog-card-block',
                'title' => __('ブログカードブロック', 'blog-card-block'),
                'icon'  => 'admin-links',
            ),
        )
    );
}
add_filter('block_categories_all', 'bcb_add_block_categories', 10, 1);

function bcb_render_callback($attributes, $content) {
    $url = isset($attributes['url']) ? esc_url($attributes['url']) : '';
    
    if (empty($url)) {
        return '';
    }
    

    $settings = get_option('bcb_settings', array());
    

    $showThumbnail = isset($settings['default_show_thumbnail']) ? (bool)$settings['default_show_thumbnail'] : true;
    $showExcerpt = isset($settings['default_show_excerpt']) ? (bool)$settings['default_show_excerpt'] : true;
    $showSiteInfo = isset($settings['default_show_site_info']) ? (bool)$settings['default_show_site_info'] : true;
    $thumbnailPosition = isset($settings['default_thumbnail_position']) ? $settings['default_thumbnail_position'] : 'left';
    $borderStyle = isset($settings['default_border_style']) ? $settings['default_border_style'] : 'solid';
    $borderWidth = isset($settings['default_border_width']) ? intval($settings['default_border_width']) : 1;
    $borderColor = isset($settings['default_border_color']) ? $settings['default_border_color'] : '#ddd';
    $borderRadius = isset($settings['default_border_radius']) ? intval($settings['default_border_radius']) : 8;
    $backgroundColor = isset($settings['default_background_color']) ? $settings['default_background_color'] : '#ffffff';
    $width = isset($settings['default_width']) ? intval($settings['default_width']) : 100;
    $alignment = isset($settings['default_alignment']) ? $settings['default_alignment'] : 'left';
    
    $cache_key = 'bcb_card_' . md5($url);
    $card_data = get_transient($cache_key);
    
    if (false === $card_data) {
        $card_data = bcb_get_url_data($url);
        if ($card_data) {
            set_transient($cache_key, $card_data, HOUR_IN_SECONDS * 24);
        }
    }
    
    if (!$card_data) {
        return sprintf(
            '<div class="bcb-card bcb-error">
                <p>%s</p>
                <a href="%s" target="_blank" rel="noopener noreferrer">%s</a>
            </div>',
            __('リンク情報を取得できませんでした。', 'blog-card-block'),
            esc_url($url),
            esc_url($url)
        );
    }
    

    $title = isset($card_data['title']) ? esc_html($card_data['title']) : '';
    $excerpt = isset($card_data['excerpt']) ? esc_html($card_data['excerpt']) : '';
    $thumbnail = isset($card_data['thumbnail']) ? $card_data['thumbnail'] : '';
    $site_name = isset($card_data['site_name']) ? esc_html($card_data['site_name']) : '';
    
    $title_length = isset($settings['title_max_lines']) ? intval($settings['title_max_lines']) : 2;
    $excerpt_length = isset($settings['excerpt_max_lines']) ? intval($settings['excerpt_max_lines']) : 3;
    
    $style = sprintf(
        'max-width: %s%%; margin-left: %s; margin-right: %s; border: %spx %s %s; border-radius: %spx; background-color: %s;',
        esc_attr($width),
        $alignment === 'center' || $alignment === 'right' ? 'auto' : '0',
        $alignment === 'center' || $alignment === 'left' ? 'auto' : '0',
        esc_attr($borderWidth),
        esc_attr($borderStyle),
        esc_attr($borderColor),
        esc_attr($borderRadius),
        esc_attr($backgroundColor)
    );
    
    $card_classes = array(
        'bcb-card',
        'bcb-thumbnail-' . esc_attr($thumbnailPosition),
        'bcb-align-' . esc_attr($alignment),
    );
    
    if (!$showThumbnail) {
        $card_classes[] = 'bcb-no-thumbnail';
    }
    
    if (!$showExcerpt) {
        $card_classes[] = 'bcb-no-excerpt';
    }
    
    if (!$showSiteInfo) {
        $card_classes[] = 'bcb-no-site-info';
    }
    
    $output = sprintf(
        '<div class="%s" style="%s">
            <a href="%s" target="_blank" rel="noopener noreferrer" class="bcb-card-link">',
        esc_attr(implode(' ', $card_classes)),
        $style,
        esc_url($url)
    );
    

if ($showThumbnail && !empty($thumbnail)) {
    $output .= sprintf(
        '<div class="bcb-thumbnail" style="aspect-ratio: 1/1;">
            <img src="%s" alt="%s" />
        </div>',
        esc_url($thumbnail),
        esc_attr($title)
    );
}
    
    $output .= '<div class="bcb-content">';
    
    $title_style = sprintf(
        'display: -webkit-box; -webkit-line-clamp: %d; -webkit-box-orient: vertical; overflow: hidden;',
        esc_attr($title_length)
    );
    
    $output .= sprintf(
        '<div class="bcb-title" style="%s">%s</div>',
        $title_style,
        $title
    );
    
    if ($showExcerpt && !empty($excerpt)) {
        $excerpt_style = sprintf(
            'display: -webkit-box; -webkit-line-clamp: %d; -webkit-box-orient: vertical; overflow: hidden;',
            esc_attr($excerpt_length)
        );
        
        $output .= sprintf(
            '<div class="bcb-excerpt" style="%s">%s</div>',
            $excerpt_style,
            $excerpt
        );
    }
    

    if ($showSiteInfo) {
        $output .= '<div class="bcb-site-info">';
        
        if (!empty($site_name)) {
            $output .= sprintf('<div class="bcb-site-name">%s</div>', $site_name);
        }
        
        $output .= sprintf('<div class="bcb-site-url">%s</div>', parse_url($url, PHP_URL_HOST));
        
        $output .= '</div>';
    }
    
    $output .= '</div></a></div>';
    
    return $output;
}


function normalize_image_url($img_url, $base_url) {
    if (empty($img_url)) {
        return '';
    }
    

    if (preg_match('/^https?:\/\//i', $img_url)) {
        return $img_url;
    }
    

    if (substr($img_url, 0, 5) === 'data:') {
        return $img_url;
    }
    

    $parsed_base = parse_url($base_url);
    $base = $parsed_base['scheme'] . '://' . $parsed_base['host'];
    

    if (substr($img_url, 0, 1) === '/') {

        return $base . $img_url;
    } else {

        $path = isset($parsed_base['path']) ? $parsed_base['path'] : '';

        if (!empty($path) && substr($path, -1) !== '/') {
            $path = dirname($path) . '/';
        }
        

        if (!empty($path) && substr($path, -1) !== '/') {
            $path .= '/';
        }
        
        return $base . $path . $img_url;
    }
}

function bcb_get_url_data($url) {
    $response = wp_remote_get($url, array(
        'timeout' => 15,
        'user-agent' => 'Mozilla/5.0 (compatible; BlogCardBlock/1.0; +http://wordpress.org)',
    ));
    
    if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
        return false;
    }
    
    $html = wp_remote_retrieve_body($response);
    
    if (empty($html)) {
        return false;
    }
    
    libxml_use_internal_errors(true);
    $doc = new DOMDocument();
    @$doc->loadHTML(mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8'));
    $xpath = new DOMXPath($doc);
    

    $title = '';
    $title_node = $xpath->query('//meta[@property="og:title"]/@content');
    if ($title_node && $title_node->length > 0) {
        $title = $title_node->item(0)->nodeValue;
    } else {
        $title_node = $xpath->query('//title');
        if ($title_node && $title_node->length > 0) {
            $title = $title_node->item(0)->nodeValue;
        }
    }
    

    $excerpt = '';
    $excerpt_node = $xpath->query('//meta[@property="og:description"]/@content');
    if ($excerpt_node && $excerpt_node->length > 0) {
        $excerpt = $excerpt_node->item(0)->nodeValue;
    } else {
        $excerpt_node = $xpath->query('//meta[@name="description"]/@content');
        if ($excerpt_node && $excerpt_node->length > 0) {
            $excerpt = $excerpt_node->item(0)->nodeValue;
        }
    }
    

    $thumbnail = '';
    
    // 1. OpenGraph画像
    $og_image_node = $xpath->query('//meta[@property="og:image"]/@content');
    if ($og_image_node && $og_image_node->length > 0) {
        $thumbnail = $og_image_node->item(0)->nodeValue;
        $thumbnail = normalize_image_url($thumbnail, $url);
    }
    

    if (empty($thumbnail)) {
        $twitter_image_node = $xpath->query('//meta[@name="twitter:image"]/@content');
        if ($twitter_image_node && $twitter_image_node->length > 0) {
            $thumbnail = $twitter_image_node->item(0)->nodeValue;
            $thumbnail = normalize_image_url($thumbnail, $url);
        }
    }
    

    if (empty($thumbnail)) {
        $img_nodes = $xpath->query('//img[@src]');
        if ($img_nodes && $img_nodes->length > 0) {

            $images = array();
            $min_width = 150; 
            $min_height = 150; 
            
            for ($i = 0; $i < $img_nodes->length; $i++) {
                $img = $img_nodes->item($i);
                $img_src = $img->getAttribute('src');
                

                if (strpos($img_src, 'icon') !== false ||
                    strpos($img_src, 'logo') !== false ||
                    strpos($img_src, 'avatar') !== false ||
                    strpos($img_src, 'banner') !== false ||
                    strpos($img_src, 'button') !== false) {
                    continue;
                }
                

                $width = $img->getAttribute('width') ? (int)$img->getAttribute('width') : 0;
                $height = $img->getAttribute('height') ? (int)$img->getAttribute('height') : 0;
                

                if ($width === 0 || $height === 0) {
                    $style = $img->getAttribute('style');
                    if (!empty($style)) {
                        preg_match('/width\s*:\s*(\d+)px/i', $style, $width_matches);
                        preg_match('/height\s*:\s*(\d+)px/i', $style, $height_matches);
                        
                        if (!empty($width_matches[1])) {
                            $width = (int)$width_matches[1];
                        }
                        if (!empty($height_matches[1])) {
                            $height = (int)$height_matches[1];
                        }
                    }
                }
                

                if ($width >= $min_width && $height >= $min_height) {
                    $images[] = array(
                        'src' => $img_src,
                        'width' => $width,
                        'height' => $height,
                        'score' => $width * $height, 
                    );
                } elseif (empty($width) && empty($height)) {

                    $images[] = array(
                        'src' => $img_src,
                        'width' => 0,
                        'height' => 0,
                        'score' => 1,
                    );
                }
            }
            

            usort($images, function($a, $b) {
                return $b['score'] - $a['score'];
            });
            

            if (!empty($images)) {
                $thumbnail = normalize_image_url($images[0]['src'], $url);
            }
        }
    }
    

    if (empty($thumbnail)) {

        $apple_icon_node = $xpath->query('//link[@rel="apple-touch-icon"]/@href');
        if ($apple_icon_node && $apple_icon_node->length > 0) {
            $thumbnail = $apple_icon_node->item(0)->nodeValue;
            $thumbnail = normalize_image_url($thumbnail, $url);
        } else {

            $icon_192_node = $xpath->query('//link[@rel="icon" and @sizes="192x192"]/@href');
            if ($icon_192_node && $icon_192_node->length > 0) {
                $thumbnail = $icon_192_node->item(0)->nodeValue;
                $thumbnail = normalize_image_url($thumbnail, $url);
            } else {

                $favicon_node = $xpath->query('//link[@rel="shortcut icon" or @rel="icon"]/@href');
                if ($favicon_node && $favicon_node->length > 0) {
                    $thumbnail = $favicon_node->item(0)->nodeValue;
                    $thumbnail = normalize_image_url($thumbnail, $url);
                } else {

                    $parsed_url = parse_url($url);
                    $favicon_url = $parsed_url['scheme'] . '://' . $parsed_url['host'] . '/favicon.ico';
                    

                    $favicon_response = wp_remote_head($favicon_url, array('timeout' => 5));
                    if (!is_wp_error($favicon_response) && 200 === wp_remote_retrieve_response_code($favicon_response)) {
                        $thumbnail = $favicon_url;
                    }
                }
            }
        }
    }
    

    $site_name = '';
    $site_name_node = $xpath->query('//meta[@property="og:site_name"]/@content');
    if ($site_name_node && $site_name_node->length > 0) {
        $site_name = $site_name_node->item(0)->nodeValue;
    } else {

        $site_name = parse_url($url, PHP_URL_HOST);
    }
    
    if (empty($title)) {
        return false;
    }
    
    return array(
        'title' => $title,
        'excerpt' => $excerpt,
        'thumbnail' => $thumbnail,
        'site_name' => $site_name,
    );
}

function bcb_enqueue_assets() {
    wp_enqueue_style('bcb-style', BCB_PLUGIN_URL . 'build/style-index.css');
    $settings = get_option('bcb_settings', array());
    wp_add_inline_style('bcb-style', bcb_generate_custom_css($settings));
}
add_action('wp_enqueue_scripts', 'bcb_enqueue_assets');

function bcb_generate_custom_css($settings) {
    $css = '';
    
    if (!empty($settings['title_font_size'])) {
        $css .= '.bcb-card .bcb-title { font-size: ' . esc_attr($settings['title_font_size']) . 'px; }';
    }
    
    if (!empty($settings['title_font_weight'])) {
        $css .= '.bcb-card .bcb-title { font-weight: ' . esc_attr($settings['title_font_weight']) . '; }';
    }
    
    if (!empty($settings['title_color'])) {
        $css .= '.bcb-card .bcb-title { color: ' . esc_attr($settings['title_color']) . '; }';
    }
    
    if (!empty($settings['excerpt_font_size'])) {
        $css .= '.bcb-card .bcb-excerpt { font-size: ' . esc_attr($settings['excerpt_font_size']) . 'px; }';
    }
    
    if (!empty($settings['excerpt_font_weight'])) {
        $css .= '.bcb-card .bcb-excerpt { font-weight: ' . esc_attr($settings['excerpt_font_weight']) . '; }';
    }
    
    if (!empty($settings['excerpt_color'])) {
        $css .= '.bcb-card .bcb-excerpt { color: ' . esc_attr($settings['excerpt_color']) . '; }';
    }
    
    if (!empty($settings['site_name_font_size'])) {
        $css .= '.bcb-card .bcb-site-name { font-size: ' . esc_attr($settings['site_name_font_size']) . 'px; }';
    }
    
    if (!empty($settings['site_name_font_weight'])) {
        $css .= '.bcb-card .bcb-site-name { font-weight: ' . esc_attr($settings['site_name_font_weight']) . '; }';
    }
    
    if (!empty($settings['site_name_color'])) {
        $css .= '.bcb-card .bcb-site-name { color: ' . esc_attr($settings['site_name_color']) . '; }';
    }
    
    if (!empty($settings['site_url_font_size'])) {
        $css .= '.bcb-card .bcb-site-url { font-size: ' . esc_attr($settings['site_url_font_size']) . 'px; }';
    }
    
    if (!empty($settings['site_url_font_weight'])) {
        $css .= '.bcb-card .bcb-site-url { font-weight: ' . esc_attr($settings['site_url_font_weight']) . '; }';
    }
    
    if (!empty($settings['site_url_color'])) {
        $css .= '.bcb-card .bcb-site-url { color: ' . esc_attr($settings['site_url_color']) . '; }';
    }
    

    $css .= '@media screen and (max-width: 600px) {
        .bcb-card.bcb-thumbnail-right .bcb-thumbnail {
            order: -1;
        }
        .bcb-card.bcb-thumbnail-left .bcb-thumbnail {
            order: -1;
        }
    }';
    
    return $css;
}

function bcb_register_rest_route() {
    register_rest_route('blog-card-block/v1', '/url-data', array(
        'methods' => 'GET',
        'callback' => 'bcb_get_url_data_api',
        'permission_callback' => function() {
            return current_user_can('edit_posts');
        },
    ));
}
add_action('rest_api_init', 'bcb_register_rest_route');

function bcb_get_url_data_api($request) {
    $url = esc_url_raw($request->get_param('url'));
    
    if (empty($url)) {
        return new WP_Error('invalid_url', __('URLが無効です。', 'blog-card-block'), array('status' => 400));
    }
    
    $cache_key = 'bcb_card_' . md5($url);
    $card_data = get_transient($cache_key);
    
    if (false === $card_data) {
        $card_data = bcb_get_url_data($url);
        if ($card_data) {
            set_transient($cache_key, $card_data, HOUR_IN_SECONDS * 24);
        }
    }
    
    if (!$card_data) {
        return new WP_Error('fetch_failed', __('リンク情報を取得できませんでした。', 'blog-card-block'), array('status' => 404));
    }
    
    return $card_data;
}

include_once BCB_PLUGIN_DIR . 'includes/settings.php';